/*
This file is an example from https://learngameprog.com

MIT License

Copyrights © 2019, Philippe-Henri Gosselin.

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the “Software”), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

The Software is provided “as is”, without warranty of any kind, express or 
implied, including but not limited to the warranties of merchantability, fitness 
for a particular purpose and noninfringement. In no event shall the authors or 
copyright holders be liable for any claim, damages or other liability, whether 
in an action of contract, tort or otherwise, arising from, out of or in 
connection with the software or the use or other dealings in the Software.

Except as contained in this notice, the name of Philippe-Henri Gosselin shall 
not be used in advertising or otherwise to promote the sale, use or other 
dealings in this Software without prior written authorization from 
Philippe-Henri Gosselin.

*/
package com.learngameprog.awtfacade04;

import com.learngameprog.awtfacade04.awt.AWTGUIFacade;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;

public class Main {
    
    public static void run(GUIFacade gui) 
    {        
        int scale = 2;
        int tileWidth = 16;
        int tileHeight = 16;
        int levelWidth = 17;
        int levelHeight = 17;
        
        Layer backgroundLayer = gui.createLayer();
        backgroundLayer.setTileSize(tileWidth,tileHeight);
        backgroundLayer.setTexture("advancewars-tileset1.png");
        backgroundLayer.setSpriteCount(levelWidth*levelHeight);
        for(int y=0;y<levelHeight;y++) {
            for(int x=0;x<levelWidth;x++) {
                int index = x + y * levelWidth;
                backgroundLayer.setSpriteLocation(index, new Rectangle(scale*x*tileWidth, scale*y*tileHeight, scale*tileWidth, scale*tileHeight));
                backgroundLayer.setSpriteTexture(index, new Rectangle(new Point(7,0), new Dimension(1,1)));
            }
        }

        Layer groundLayer = gui.createLayer();
        groundLayer.setTileSize(tileWidth,tileHeight);
        groundLayer.setTexture("advancewars-tileset2.png");
        groundLayer.setSpriteCount(2);
        groundLayer.setSpriteLocation(0, new Rectangle(scale*8*tileWidth, scale*6*tileHeight, scale*tileWidth, scale*2*tileHeight));
        groundLayer.setSpriteTexture(0, new Rectangle(new Point(0,2), new Dimension(1,2)));
        groundLayer.setSpriteLocation(1, new Rectangle(scale*8*tileWidth, scale*7*tileHeight, scale*tileWidth, scale*2*tileHeight));
        groundLayer.setSpriteTexture(1, new Rectangle(new Point(0,4), new Dimension(1,2)));
        

        gui.createWindow("AWT Graphic User Interface Facade",
            scale*tileWidth*levelWidth,
            scale*tileHeight*levelHeight);

        int fps = 60;
        long nanoPerFrame = (long) (1000000000.0 / fps);
        long lastTime = 0;
        
        while(!gui.isClosingRequested()) {
            long nowTime = System.nanoTime();
            if ((nowTime-lastTime) < nanoPerFrame) {
                continue;
            }
            lastTime = nowTime;            
            
            if (gui.beginPaint()) {
                gui.drawLayer(backgroundLayer);
                gui.drawLayer(groundLayer);
                gui.endPaint();
            }
            
            long elapsed = System.nanoTime() - lastTime;
            long milliSleep = (nanoPerFrame - elapsed) / 1000000;
            if (milliSleep > 0) {
                try {
                    Thread.sleep (milliSleep);
                } catch (InterruptedException ex) {
                    ex.printStackTrace();
                }
            }               
        }      
        gui.dispose();
    }
    
    public static void main(String args[]) {
        GUIFacade gui = new AWTGUIFacade();
        run(gui);
    }        
}
