/*
This file is an example from https://learngameprog.com

MIT License

Copyrights © 2019, Philippe-Henri Gosselin.

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the “Software”), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

The Software is provided “as is”, without warranty of any kind, express or 
implied, including but not limited to the warranties of merchantability, fitness 
for a particular purpose and noninfringement. In no event shall the authors or 
copyright holders be liable for any claim, damages or other liability, whether 
in an action of contract, tort or otherwise, arising from, out of or in 
connection with the software or the use or other dealings in the Software.

Except as contained in this notice, the name of Philippe-Henri Gosselin shall 
not be used in advertising or otherwise to promote the sale, use or other 
dealings in this Software without prior written authorization from 
Philippe-Henri Gosselin.

*/
package com.learngameprog.awtfacade04.awt;

import com.learngameprog.awtfacade04.GUIFacade;
import com.learngameprog.awtfacade04.Image;
import com.learngameprog.awtfacade04.Layer;
import java.awt.Color;
import java.awt.Graphics;

public class AWTGUIFacade implements GUIFacade {

    private AWTWindow window;
    
    private boolean closingRequested = false;
        
    private Graphics graphics;

    @Override
    public void createWindow(String title, int width, int height) {
        window = new AWTWindow(this);
        window.init(title,width,height);
        window.setLocationRelativeTo(null);
        window.setVisible(true);
        window.createBufferStrategy(2);
    }
    
    @Override
    public void setClosingRequested(boolean b) {
        closingRequested = b;
    }
    
    @Override
    public boolean isClosingRequested() {
        return closingRequested;
    }
    
    @Override
    public void dispose() {
        window.dispose();
    }
    
    @Override
    public Image createImage(String fileName) {
        AWTImage image = new AWTImage();
        image.loadImage(fileName);
        return image;
    }    
    
    @Override
    public boolean beginPaint() {
        graphics = window.createGraphics();
        if (graphics == null)
            return false;
        graphics.setColor(Color.black);
        graphics.fillRect(0, 0, window.getWidth(), window.getHeight());
        return true;
    }

    @Override
    public void drawLine(int x1, int y1, int x2, int y2) {
        graphics.setColor(Color.white);
        graphics.drawLine(x1, y1, x2, y2);
    }
    
    @Override
    public void drawImage(Image image, int x, int y) {
        if (!(image instanceof AWTImage))
            throw new IllegalArgumentException("Invalid image type");
        AWTImage awtImage = (AWTImage) image;
        awtImage.draw(graphics,x,y);
    }    
    
    @Override
    public void endPaint() {
        graphics.dispose();
        window.switchBuffers();
    }
    
    @Override
    public Layer createLayer() {
        return new AWTLayer();
    }
    
    @Override
    public void drawLayer(Layer layer) {
        if (graphics == null) {
            return;
        }
        if (layer == null)
            throw new IllegalArgumentException("No layer");
        if (!(layer instanceof AWTLayer))
            throw new IllegalArgumentException("Invalid layer type");
        AWTLayer awtLayer = (AWTLayer) layer;
        awtLayer.draw(graphics);
    }

}
