/*
This file is an example from https://learngameprog.com

MIT License

Copyrights © 2019, Philippe-Henri Gosselin.

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the “Software”), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

The Software is provided “as is”, without warranty of any kind, express or 
implied, including but not limited to the warranties of merchantability, fitness 
for a particular purpose and noninfringement. In no event shall the authors or 
copyright holders be liable for any claim, damages or other liability, whether 
in an action of contract, tort or otherwise, arising from, out of or in 
connection with the software or the use or other dealings in the Software.

Except as contained in this notice, the name of Philippe-Henri Gosselin shall 
not be used in advertising or otherwise to promote the sale, use or other 
dealings in this Software without prior written authorization from 
Philippe-Henri Gosselin.

*/
package com.learngameprog.awtfacade04.awt;

import com.learngameprog.awtfacade04.Layer;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.io.IOException;
import javax.imageio.ImageIO;

public class AWTLayer implements Layer {

    private int tileWidth;

    private int tileHeight;

    private BufferedImage texture;

    private int textureWidth;

    private int textureHeight;

    private Rectangle[] textures;
    
    private Rectangle[] locations;

    @Override
    public int getTileWidth() {
        return tileWidth;
    }

    @Override
    public int getTileHeight() {
        return tileHeight;
    }

    @Override
    public int getTextureWidth() {
        return textureWidth;
    }

    @Override
    public int getTextureHeight() {
        return textureHeight;
    }
        
    @Override
    public void setTileSize(int width, int height) {
        this.tileWidth = width;
        this.tileHeight = height;
    }

    @Override
    public void setTexture(String fileName) {
        if (tileWidth == 0 || tileHeight == 0) {
            throw new RuntimeException("Tile size is not defined");
        }
        try {
            texture = ImageIO.read(this.getClass().getClassLoader().getResource(fileName));
        } catch (IOException ex) {
            throw new RuntimeException("Error when reading "+fileName);
        }
        textureWidth = texture.getWidth() / tileWidth;
        textureHeight = texture.getHeight() / tileHeight;
    }

    @Override
    public void setSpriteCount(int count) {
        textures = new Rectangle[count];
        locations = new Rectangle[count];
    }

    @Override
    public void setSpriteTexture(int index, Rectangle tile) {
        if (textures == null) {
            throw new RuntimeException("Sprites are not defined");
        }
        if (index < 0 || index >= textures.length) {
            throw new IllegalArgumentException("Invalid sprite index "+ index);
        }
        if (tile.x >= 0 && tile.x < textureWidth && tile.y >= 0 && tile.y < textureHeight) {
            textures[index] = tile;
        }
    }
    
    @Override
    public void setSpriteLocation(int index, Rectangle location) {
        if (locations == null || index < 0 || index >= locations.length)
            return;
        locations[index] = location;
    }

    public void draw(Graphics graphics) {
        if (texture == null) {
            throw new RuntimeException("Texture is not loaded");
        }
        if (locations == null) {
            throw new RuntimeException("Sprites are not defined");
        }
        if (tileWidth == 0 || tileHeight == 0) {
            throw new RuntimeException("Tile size is not defined");
        }
        for (int i = 0; i < locations.length; i++) {
            if (textures[i] != null) {
                graphics.drawImage(texture, 
                    locations[i].x, locations[i].y, locations[i].x + locations[i].width, locations[i].y + locations[i].height, 
                    textures[i].x * tileWidth, textures[i].y * tileHeight, (textures[i].x+textures[i].width) * tileWidth, (textures[i].y+textures[i].height) * tileHeight, null);
            }
        }
    }

}
